#!/usr/bin/env python
import user
import maker

import os

class Make(maker.Make):
  def setupDependencies(self, sourceDB):
    maker.Make.setupDependencies(self, sourceDB)
    #sourceDB.addDependency(os.path.join('ParMETISLib', 'kmerge.c'), 'parmetis.h')
    return

  def setupConfigure(self, framework):
    shouldRun = maker.Make.setupConfigure(self, framework)
    framework.header  = os.path.join('ParMETISLib','configureheader.h')
    framework.cHeader = None
    return shouldRun

  def configure(self, builder):
    import sys

    if 'PETSC_DIR' in self.argDB:
      dir = self.argDB['PETSC_DIR']
    elif 'PETSC_DIR' in os.environ:
      dir = os.environ['PETSC_DIR']
    else:
      raise RuntimeError('Could not find PETSC_DIR for MPI configure')
    sys.path.append(os.path.join(dir, 'python'))
    framework = maker.Make.configure(self, builder)
    self.mpi = framework.require('PETSc.packages.MPI', None)
    self.libraries = framework.require('config.libraries', None)
    self.setCompilers = framework.require('config.setCompilers', None)
    return

  def buildDirectories(self, builder):
    if not self.prefix is None:
      self.incDir = os.path.abspath(os.path.join(self.prefix, 'include'))
      self.libDir = os.path.abspath(os.path.join(self.prefix, 'lib'))
      self.binDir = os.path.abspath(os.path.join(self.prefix, 'bin'))
    else:
      self.incDir = os.path.abspath('include')
      self.libDir = os.path.abspath('lib')
      self.binDir = os.path.abspath('bin')
    for dir in [self.incDir, self.libDir, self.binDir]:
      if not os.path.isdir(dir):
        os.makedirs(dir)
    return

  def buildMetis(self, builder):
    '''Builds the Metis serial partitioning library'''
    builder.loadConfiguration('Metis Library')
    config = builder.pushConfiguration('Metis Library')
    builder.getCompilerObject().includeDirectories.add('METISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getSharedLinkerObject().libraries.update(self.mpi.lib)
    for source in [f for f in os.listdir('METISLib') if os.path.splitext(f)[1] == '.c']:
      self.builder.compile([os.path.join('METISLib', source)])
    self.metisLibrary = os.path.join(self.libDir, 'libmetis.'+self.setCompilers.sharedLibraryExt)
    self.builder.link(config.outputFiles['ELF'], self.metisLibrary, shared = 1)
    builder.popConfiguration()
    builder.saveConfiguration('Metis Library')
    return

  def buildParMetis(self, builder):
    '''Builds the ParMetis parallel partitioning library'''
    builder.loadConfiguration('ParMetis Library')
    config = builder.pushConfiguration('ParMetis Library')
    builder.getCompilerObject().includeDirectories.add('.')
    builder.getCompilerObject().includeDirectories.add('ParMETISLib')
    builder.getCompilerObject().includeDirectories.add('METISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getSharedLinkerObject().libraries.update(self.mpi.lib)
    for source in [f for f in os.listdir('ParMETISLib') if os.path.splitext(f)[1] == '.c']:
      self.builder.compile([os.path.join('ParMETISLib', source)])
    self.parmetisLibrary = os.path.join(self.libDir, 'libparmetis.'+self.setCompilers.sharedLibraryExt)
    self.builder.link(config.outputFiles['ELF'], self.parmetisLibrary, shared = 1)
    builder.popConfiguration()
    builder.saveConfiguration('ParMetis Library')
    return

  def buildPrograms(self, builder):
    '''Builds the test programs'''
    builder.loadConfiguration('Test Binary')
    builder.pushConfiguration('Test Binary')
    builder.getCompilerObject().includeDirectories.add('.')
    builder.getCompilerObject().includeDirectories.add('Programs')
    builder.getCompilerObject().includeDirectories.add('ParMETISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getLinkerObject().libraries.add(self.parmetisLibrary)
    builder.getLinkerObject().libraries.add(self.metisLibrary)
    if self.libraries.math:
      builder.getLinkerObject().libraries.update(self.libraries.math)
    builder.getLinkerObject().libraries.updateDuplicate(self.mpi.lib)
    sources = ['ptest.c', 'io.c', 'adaptgraph.c']
    for source in sources:
      self.builder.compile([os.path.join('Programs', source)])
    self.builder.link([os.path.join('Programs', self.builder.getCompilerTarget(source)) for source in sources], os.path.join(self.binDir, 'ptest'))
    sources = ['mtest.c', 'io.c']
    for source in sources:
      self.builder.compile([os.path.join('Programs', source)])
    self.builder.link([os.path.join('Programs', self.builder.getCompilerTarget(source)) for source in sources], os.path.join(self.binDir, 'mtest'))
    builder.popConfiguration()
    builder.saveConfiguration('Test Binary')
    return

  def updateDependencies(self, sourceDB):
    sourceDB.updateSource('parmetis.h')
    maker.Make.updateDependencies(self, sourceDB)
    return

  def build(self, builder, setupOnly = 0):
    self.buildDirectories(builder)
    self.buildMetis(builder)
    self.buildParMetis(builder)
    self.buildPrograms(builder)
    return

  def install(self, builder, argDB):
    '''Installs the header'''
    import shutil

    maker.Make.install(self, builder, argDB)
    header = os.path.join(self.incDir, 'parmetis.h')
    if os.path.isfile(header):
      os.remove(header)
    shutil.copy('parmetis.h', self.incDir)
    os.chmod(header, 0664)
    return

if __name__ == '__main__':
  Make().run()
